package com.jpattern.orm.dialect;

import java.sql.Statement;

/**
 * 
 * @author Francesco Cina
 *
 * 28/giu/2011
 * 
 * This class take care of the small differences between different database implementations
 */
public interface DBProfile  {

    /**
     * Return true if the DB has sequence support and the method getGeneratedKeys of the  {@link Statement}
     * correctly return the sequence value generated using a sequence in an insert query.
     * @return
     */
    boolean isSequenceSupport();

    /**
     * Return true if the DB has autogenerated key features.
     * @return
     */
    boolean isAutogeneratedKeySupport();

    /**
     * Return the semantic of the DB to call a sequence inside an insert sql query.
     * E.G.
     * - For Oracle DB the returned String is:
     *   sequenceName + ".nextval"
     * - For HSQLDB the returned value is:
     *   "NEXT VALUE FOR " + sequenceName
     * @param sequenceName
     * @return
     */
    String insertQuerySequence(String sequenceName);

}
