package com.jpattern.orm.dialect;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import com.jpattern.orm.util.StringUtil;

/**
 * Return the type of underlying DB
 * @author ufo
 *
 */
public class DetermineDBType {

    private final Map<String, DBType> dbProductNameMap = new HashMap<String, DBType>();

    public DetermineDBType() {

        dbProductNameMap.put("Derby", DBType.DERBY); //$NON-NLS-1$

        dbProductNameMap.put("H2", DBType.H2); //$NON-NLS-1$

        dbProductNameMap.put("HSQL", DBType.HSQLDB); //$NON-NLS-1$

        dbProductNameMap.put("Oracle", DBType.ORACLE); //$NON-NLS-1$

        dbProductNameMap.put("Mysql", DBType.MYSQL); //$NON-NLS-1$
    }

    public DBType determineDBType(final String driverName, final String URL, final String databaseProductName) {
        for (Entry<String, DBType> entry : dbProductNameMap.entrySet()) {
            if ((databaseProductName!=null) && StringUtil.containsIgnoreCase(databaseProductName, entry.getKey())) {
                return entry.getValue();
            }
        }
        return DBType.UNKNOWN;
    }

}
