package com.jpattern.orm.mapper;

import java.util.Map;
import java.util.HashMap;

import com.jpattern.orm.JPO;
import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.persistor.type.TypeFactory;


/**
 * 
 * @author Francesco Cina
 *
 * 22/mag/2011
 */
public class OrmClassToolMap implements IOrmClassToolMap {

    private final TypeFactory typeFactory = new TypeFactory();
    private final Map<Class<?>, IOrmClassTool<?>> classToolMap = new HashMap<Class<?>, IOrmClassTool<?>>();
    private final JPO jpOrm;

    public OrmClassToolMap(final JPO jpOrm) {
        this.jpOrm = jpOrm;
    }

    @Override
    public <T> void put(final Class<T> clazz, final IOrmClassTool<T> ormClassTool) {
        classToolMap.put(clazz, ormClassTool);
    }

    @Override
    public boolean containsTool(final Class<?> clazz) {
        return classToolMap.containsKey(clazz);
    }

    @SuppressWarnings("unchecked")
    @Override
    public <T> IOrmClassTool<T> getOrmClassTool(final Class<T> clazz) throws OrmException {
        IOrmClassTool<?> ormClazzTool = classToolMap.get(clazz);
        if (ormClazzTool==null) {
            jpOrm.register(clazz);
            ormClazzTool = classToolMap.get(clazz);
        }
        return (IOrmClassTool<T>) ormClazzTool;
    }

    @Override
    public TypeFactory getTypeFactory() {
        return typeFactory;
    }

}
