package com.jpattern.orm.mapper.clazz;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.jpattern.orm.annotation.Version;
import com.jpattern.orm.annotation.table.TableInfo;
import com.jpattern.orm.exception.OrmConfigurationException;
import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.mapper.relation.ClassFKMapImpl;
import com.jpattern.orm.mapper.relation.FKClassField;
import com.jpattern.orm.mapper.relation.RelationOuterFK;

/**
 * 
 * @author Francesco Cina
 *
 * 22/mag/2011
 */
public class ClassMap<BEAN> implements IClassMap<BEAN> {

    private final TableInfo tableInfo;
    private final Class<BEAN> mappedClass;
    private final ClassFKMapImpl<BEAN> classFKMap;
    private final Map<String, ClassField<BEAN, ?>> fieldClassMapByJavaName = new HashMap<String, ClassField<BEAN,?>>();
    private String[] allColumnJavaNames = new String[0];
    private String[] allNotGeneratedColumnJavaNames = new String[0];
    private String[] primaryKeyColumnJavaNames = new String[0];
    private String[] primaryKeyAndVersionColumnJavaNames = new String[0];
    private String[] notPrimaryKeyColumnJavaNames = new String[0];
    private String[] allGeneratedColumnJavaNames = new String[0];
    private String[] allGeneratedColumnDBNames = new String[0];
    private boolean versionGenerator = false;
    private final List<RelationOuterFK<BEAN, ?, ?>> outerRelations = new ArrayList<RelationOuterFK<BEAN, ?, ?>>();
    private final List<FKClassField<BEAN, ?>> innerRelations = new ArrayList<FKClassField<BEAN,?>>();

    public ClassMap(final Class<BEAN> mappedClass, final TableInfo tableInfo) {
        this.mappedClass = mappedClass;
        this.tableInfo = tableInfo;
        classFKMap = new ClassFKMapImpl<BEAN>(mappedClass);
    }

    @Override
    public Class<BEAN> getMappedClass() {
        return this.mappedClass;
    }

    @Override
    public <P> ClassField<BEAN, P> getClassFieldByJavaName(final String javaName) {
        if (this.fieldClassMapByJavaName.containsKey(javaName)) {
            return (ClassField<BEAN, P>) this.fieldClassMapByJavaName.get(javaName);
        }
        throw new OrmException("Property with name [" + javaName + "] not found in class " + this.mappedClass); //$NON-NLS-1$ //$NON-NLS-2$
    }

    public <P> void addClassField(final ClassField<BEAN, P> classField) {
        this.fieldClassMapByJavaName.put(classField.getFieldName(), classField);

        if (classField.getVersionInfo().isVersionable()) {
            if (this.versionGenerator) {
                throw new OrmConfigurationException("A bean can have maximum one field annotated with @" + Version.class.getSimpleName() + ". Error in class:[" + this.getMappedClass() + "] field: [" + classField.getFieldName() + "]"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
            }
            this.versionGenerator = true;
        }

    }

    public Map<String, ClassField<BEAN, ?>> getUnmodifiableFieldClassMap() {
        return Collections.unmodifiableMap( this.fieldClassMapByJavaName );
    }

    @Override
    public String[] getAllColumnJavaNames() {
        return this.allColumnJavaNames;
    }

    @Override
    public String[] getPrimaryKeyColumnJavaNames() {
        return this.primaryKeyColumnJavaNames;
    }

    @Override
    public String[] getNotPrimaryKeyColumnJavaNames() {
        return this.notPrimaryKeyColumnJavaNames;
    }

    public void setAllColumnJavaNames(final String[] allColumnJavaNames) {
        this.allColumnJavaNames = allColumnJavaNames;
    }

    public void setPrimaryKeyColumnJavaNames(final String[] primaryKeyColumnJavaNames) {
        this.primaryKeyColumnJavaNames = primaryKeyColumnJavaNames;
    }

    public void setNotPrimaryKeyColumnJavaNames(final String[] notPrimaryKeyColumnJavaNames) {
        this.notPrimaryKeyColumnJavaNames = notPrimaryKeyColumnJavaNames;
    }

    public void setAllNotGeneratedColumnJavaNames(final String[] allNotGeneratedColumnJavaNames) {
        this.allNotGeneratedColumnJavaNames = allNotGeneratedColumnJavaNames;
    }

    @Override
    public String[] getAllNotGeneratedColumnJavaNames() {
        return this.allNotGeneratedColumnJavaNames;
    }

    public void setAllGeneratedColumnJavaNames(final String[] allGeneratedColumnJavaNames) {
        this.allGeneratedColumnJavaNames = allGeneratedColumnJavaNames;
    }

    @Override
    public String[] getAllGeneratedColumnJavaNames() {
        return this.allGeneratedColumnJavaNames;
    }

    public void setPrimaryKeyAndVersionColumnJavaNames(
            final String[] primaryKeyAndVersionColumnJavaNames) {
        this.primaryKeyAndVersionColumnJavaNames = primaryKeyAndVersionColumnJavaNames;
    }

    @Override
    public String[] getPrimaryKeyAndVersionColumnJavaNames() {
        return this.primaryKeyAndVersionColumnJavaNames;
    }

    @Override
    public TableInfo getTableInfo() {
        return this.tableInfo;
    }

    @Override
    public String[] getAllGeneratedColumnDBNames() {
        return this.allGeneratedColumnDBNames;
    }

    public void setAllGeneratedColumnDBNames(
            final String[] allGeneratedColumnDBNames) {
        this.allGeneratedColumnDBNames = allGeneratedColumnDBNames;
    }

    @Override
    public List<RelationOuterFK<BEAN, ?, ?>> getOuterRelations() {
        return outerRelations;
    }

    @Override
    public ClassFKMapImpl<BEAN> getFKs() {
        return classFKMap;
    }

    /**
     * @return
     */
    @Override
    public List<FKClassField<BEAN, ?>> getInnerRelations() {
        return innerRelations;
    }

}
