/* ----------------------------------------------------------------------------
 *     PROJECT : JPOrm
 *
 *  CREATED BY : Francesco Cina'
 *          ON : Mar 3, 2013
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.persistor;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Mar 3, 2013
 *
 * @param <BEAN> the type of the bean to manipulate
 * @param <P> the type of the bean's property to manipulate
 * @param <DB> the type of the field in the {@link PreparedStatement} and {@link ResultSet}
 */
public interface PropertyPersistor<BEAN, P, DB> {

    /**
     * @return
     */
    Class<P> propertyType();

    /**
     * @param bean
     * @return
     * @throws IllegalArgumentException
     */
    P getPropertyValueFromBean(BEAN bean) throws IllegalArgumentException;

    /**
     * @param bean
     * @param firstVersionNumber
     * @throws IllegalArgumentException
     */
    void increaseVersion(BEAN bean, boolean firstVersionNumber)
            throws IllegalArgumentException;

    /**
     * @param source
     * @param destination
     * @throws IllegalArgumentException
     */
    void clonePropertyValue(BEAN source, BEAN destination)
            throws IllegalArgumentException;

    /**
     * @param bean
     * @param rs
     * @param rsColumnIndex
     * @throws IllegalArgumentException
     * @throws SQLException
     */
    void getFromResultSet(BEAN bean, ResultSet rs, int rsColumnIndex)
            throws IllegalArgumentException, SQLException;

    /**
     * @param bean
     * @param rs
     * @throws IllegalArgumentException
     * @throws SQLException
     */
    void getFromResultSet(BEAN bean, ResultSet rs) throws IllegalArgumentException, SQLException;

    /**
     * @param bean
     * @param rs
     * @throws IllegalArgumentException
     * @throws SQLException
     */
    DB getValueFromResultSet(ResultSet rs, String fieldName) throws IllegalArgumentException, SQLException;

    boolean isInnerRelation();
}
