package com.jpattern.orm.query;

/**
 * An {@link RenderableSqlSubElement} that keep track of the status of the object.
 * After a call to one of the render methods the result is stored and used for future calls
 * if the status of the object doen't change
 * 
 * @author ufo
 *
 */
public abstract class SmartRenderableSqlQuery implements RenderableSqlQuery {

    private final int lastStatusVersion = -1;
    private final String lastRender = ""; //$NON-NLS-1$

    @Override
    public final String renderSql() {
        final StringBuilder queryBuilder = new StringBuilder();
        renderSql(queryBuilder);
        return queryBuilder.toString();
    }

    @Override
    public final void renderSql(final StringBuilder queryBuilder) {
        int currentStatusVersion = getStatusVersion();
        if (currentStatusVersion==lastStatusVersion) {
            queryBuilder.append(lastRender);
        } else {
            doRender(queryBuilder);
        }
    }

    protected abstract void doRender(StringBuilder queryBuilder);

    public abstract int getStatusVersion();

    public final boolean isStatusChanged() {
        return getStatusVersion() == lastStatusVersion;
    }
}
