package com.jpattern.orm.query;

/**
 * An {@link RenderableSqlSubElement} that keep track of the status of the object.
 * After a call to one of the render methods the result is stored and used for future calls
 * if the status of the object doen't change
 * 
 * @author ufo
 *
 */
public abstract class SmartRenderableSqlSubElement implements RenderableSqlSubElement {

    private int lastStatusVersion = -1;
    private String lastRender = ""; //$NON-NLS-1$

    @Override
    public final void renderSqlElement(final StringBuilder queryBuilder, final NameSolver nameSolver) {
        int currentStatusVersion = getElementStatusVersion();
        if (currentStatusVersion==lastStatusVersion) {
            queryBuilder.append(lastRender);
        } else {
            doElementRender(queryBuilder, nameSolver);
        }
    }

    protected abstract void doElementRender(StringBuilder queryBuilder, NameSolver nameSolver);

    public abstract int getElementStatusVersion();

    public final boolean isElementStatusChanged() {
        return getElementStatusVersion() == lastStatusVersion;
    }

    /**
     * @return
     */
    @Override
    public final String renderSqlElement(final NameSolver nameSolver) {
        StringBuilder queryBuilder = new StringBuilder();
        renderSqlElement(queryBuilder, nameSolver);
        return queryBuilder.toString();
    }
}
