package com.jpattern.orm.query.clause;

import java.util.ArrayList;
import java.util.List;

import com.jpattern.orm.query.NameSolver;
import com.jpattern.orm.query.SmartRenderableSqlSubElement;
import com.jpattern.orm.query.clause.where.EqExpressionElement;
import com.jpattern.orm.query.clause.where.ExpressionElement;

/**
 * 
 * @author Francesco Cina
 *
 * 10/lug/2011
 */
public abstract class OrmSet<T extends Set<?>> extends SmartRenderableSqlSubElement implements Set<T> {

    List<ExpressionElement> elementList = new ArrayList<ExpressionElement>();

    protected abstract T set();

    @Override
    public T eq(final String property, final Object value) {
        final ExpressionElement expressionElement = new EqExpressionElement(property, value);
        this.elementList.add(expressionElement);
        return set();
    }

    @Override
    public final void doElementRender(final StringBuilder queryBuilder, final NameSolver nameSolver) {
        boolean first = true;
        if (!this.elementList.isEmpty()) {
            queryBuilder.append("SET "); //$NON-NLS-1$
            for (final ExpressionElement expressionElement : this.elementList) {
                if (!first) {
                    queryBuilder.append(", "); //$NON-NLS-1$
                }
                expressionElement.renderSqlElement(queryBuilder, nameSolver);
                first = false;
            }
        }
    }

    @Override
    public final void appendElementValues(final List<Object> values) {
        if (!this.elementList.isEmpty()) {
            for (final ExpressionElement expressionElement : this.elementList) {
                expressionElement.appendElementValues(values);
            }
        }
    }

    @Override
    public final int getElementStatusVersion() {
        return this.elementList.size();
    }
}
