package com.jpattern.orm.query.find;

import java.util.List;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.exception.OrmNotUniqueResultException;
import com.jpattern.orm.query.LockMode;
import com.jpattern.orm.query.OrmRowMapper;
import com.jpattern.orm.query.clause.OrmOrderBy;

/**
 * 
 * @author ufo
 *
 * @param <BEAN>
 */
public class FindOrderByImpl<BEAN> extends OrmOrderBy<FindOrderBy<BEAN>> implements FindOrderBy<BEAN> {

    private final FindQuery<BEAN> findQuery;

    @Override
    public String renderSql() {
        return this.findQuery.renderSql();
    }

    @Override
    public void renderSql(final StringBuilder stringBuilder) {
        this.findQuery.renderSql(stringBuilder);
    }

    @Override
    public void appendValues(final List<Object> values) {
        this.findQuery.appendValues(values);
    }

    public FindOrderByImpl(final FindQuery<BEAN> findQuery) {
        this.findQuery = findQuery;

    }

    @Override
    public FindQuery<BEAN> query() {
        return this.findQuery;
    }

    @Override
    public FindWhere<BEAN> where() throws OrmException {
        return this.findQuery.where();
    }

    @Override
    protected FindOrderBy<BEAN> orderBy() throws OrmException {
        return this;
    }

    @Override
    public List<BEAN> getList() throws OrmException {
        return this.findQuery.getList();
    }

    @Override
    public int getRowCount() throws OrmException {
        return this.findQuery.getRowCount();
    }

    @Override
    public BEAN getUnique() throws OrmException, OrmNotUniqueResultException {
        return this.findQuery.getUnique();
    }

    @Override
    public void get(final OrmRowMapper<BEAN> srr) throws OrmException {
        this.findQuery.get(srr);
    }

    @Override
    public String renderRowCountSql() throws OrmException {
        return this.findQuery.renderRowCountSql();
    }

    @Override
    public FindQuery<BEAN> distinct(final boolean distinct) throws OrmException {
        return this.findQuery.distinct(distinct);
    }

    @Override
    public FindQuery<BEAN> lockMode(final LockMode lockMode) {
        return this.findQuery.lockMode(lockMode);
    }

    @Override
    public FindQuery<BEAN> maxRows(final int maxRows) throws OrmException {
        return this.findQuery.maxRows(maxRows);
    }

    @Override
    public FindQuery<BEAN> queryTimeout(final int queryTimeout) {
        return this.findQuery.queryTimeout(queryTimeout);
    }

    @Override
    public BEAN get() throws OrmException, OrmNotUniqueResultException {
        return this.findQuery.get();
    }

}
