package com.jpattern.orm.query.find;

import com.jpattern.orm.exception.OrmException;

/**
 * 
 * @author Francesco Cina
 *
 * 18/giu/2011
 */
public interface FindQuery<BEAN> extends FindFrom<BEAN>, FindQueryCommon<BEAN> {

    /**
     * Set the where clause.
     * @return
     */
    FindWhere<BEAN> where() throws OrmException;

    /**
     * Set the order by clause.
     * @return
     */
    FindOrderBy<BEAN> orderBy() throws OrmException;

    /**
     * Whether to fetch the bean's children
     * @param lazy
     * @return
     */
    FindQuery<BEAN> lazy(boolean lazy);

    /**
     * Activate the cache for this query.
     * @param cacheName the of the cache to use
     * @return
     */
    FindQuery<BEAN> cache(String cache);

    /**
     * The value of the Bean fields listed will not be fetched from the DB. This is useful to load only a partial Bean
     * to reduce the amount of work of the DB. Normally this is used to avoid loading LOB values when not needed.
     * @param fields
     * @return
     */
    FindQuery<BEAN> ignore(String... fields);

    /**
     * The value of the Bean fields listed will not be fetched from the DB. This is useful to load only a partial Bean
     * to reduce the amount of work of the DB. Normally this is used to avoid loading LOB values when not needed.
     * If 'ignoreFieldsCondition' is false the fields will not be ignored fetched.
     * @param fields
     * @param ignoreFieldsCondition
     * @return
     */
    FindQuery<BEAN> ignore(boolean ignoreFieldsCondition, String... fields);

}
