/* ----------------------------------------------------------------------------
 *     PROJECT : JPOrm
 *
 *  CREATED BY : Francesco Cina'
 *          ON : Mar 5, 2013
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.query.find.cache;

import java.util.ArrayList;
import java.util.List;

import com.jpattern.orm.cache.Cache;
import com.jpattern.orm.query.OrmRowMapper;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Mar 5, 2013
 *
 * @author Francesco Cina'
 * @version $Revision
 */
public class CacheStrategyImpl<BEAN> implements CacheStrategy<BEAN> {

    private final Cache cache;

    public CacheStrategyImpl(final Cache cache) {
        this.cache = cache;
    }

    @Override
    public void find(final String sql, final List<Object> values,
            final List<String> ignoredFields, final int maxRows, final OrmRowMapper<BEAN> srr,
            final CacheStrategyCallback<BEAN> cacheStrategyCallback) {
        final CacheKey key = new CacheKey(sql, values, ignoredFields, maxRows);
        List<BEAN> cached = (List<BEAN>) cache.get(key);
        if (cached!=null) {
            for (int i = 0; i < cached.size(); i++) {
                srr.read(cached.get(i), i);
            }
        } else {
            cacheStrategyCallback.doWhenNotInCache(new CacheStrategyEntry<BEAN>() {

                List<BEAN> beans = new ArrayList<BEAN>();

                @Override
                public void add(final BEAN bean) {
                    beans.add(bean);
                }

                @Override
                public void end() {
                    if(!beans.isEmpty()) {
                        cache.put(key, beans);
                    }
                }
            });
        }

    }

}
