package com.jpattern.orm.query.save;

import com.jpattern.orm.dialect.DBProfile;
import com.jpattern.orm.mapper.clazz.IClassField;

/**
 * 
 * @author Francesco Cina
 *
 * 13/giu/2011
 */
public class ColumnValueGeneratorFactory {

    private ColumnValueGeneratorFactory() {}

    public static <BEAN> AColumnValueGenerator getColumnValueGenerator(final IClassField<BEAN,? extends Object> classField, final DBProfile dbProfile, final boolean ignoreGenerator) {
        AColumnValueGenerator columnValueGenerator = new NullColumnValueGenerator(classField.getGeneratorInfo().getName(), dbProfile);
        if (!ignoreGenerator) {

            switch (classField.getGeneratorInfo().getGeneratorType()) {
            case SEQUENCE:
                columnValueGenerator = new SequenceColumnValueGenerator(classField.getGeneratorInfo().getName(), dbProfile);
                break;
            case SEQUENCE_FALLBACK_AUTOGENERATED:
                if (dbProfile.isSequenceSupport()) {
                    columnValueGenerator = new SequenceColumnValueGenerator(classField.getGeneratorInfo().getName(), dbProfile);
                } else {
                    columnValueGenerator = new AutogeneratedColumnValueGenerator(classField.getGeneratorInfo().getName(), dbProfile);
                }
                break;
            case AUTOGENERATED:
                columnValueGenerator = new AutogeneratedColumnValueGenerator(classField.getGeneratorInfo().getName(), dbProfile);
                break;
            case AUTOGENERATED_FALLBACK_SEQUENCE:
                if (dbProfile.isAutogeneratedKeySupport()) {
                    columnValueGenerator = new AutogeneratedColumnValueGenerator(classField.getGeneratorInfo().getName(), dbProfile);
                } else {
                    columnValueGenerator = new SequenceColumnValueGenerator(classField.getGeneratorInfo().getName(), dbProfile);
                }
                break;
            case NONE:
                break;
            }
        }
        columnValueGenerator.setGeneratedColumnName(classField.getColumnInfo().getDBColumnName());
        return columnValueGenerator;
    }
}