package com.jpattern.orm.session.datasource;

import javax.sql.DataSource;

import com.jpattern.orm.session.SessionProvider;
import com.jpattern.orm.session.SessionStrategy;

/**
 * 
 * @author Francesco Cina
 *
 * 21/mag/2011
 */
public class DataSourceSessionProvider extends SessionProvider {

    private final ThreadLocal<DataSourceConnection> threadLocalConnection = new ThreadLocal<DataSourceConnection>();
    private final DataSource dataSource;

    public DataSourceSessionProvider(final DataSource dataSource) {
        this.dataSource = dataSource;
    }

    @Override
    public SessionStrategy getSessionStrategy() {
        return new DataSourceSessionStrategy(this);
    }

    final DataSourceConnection getConnection(final boolean readOnly, final IConnectionCaller connectionCaller) {
        if (getLogger().isDebugEnabled()) {
            getLogger().debug("Connection asked by " + connectionCaller ); //$NON-NLS-1$
        }
        DataSourceConnection conn = threadLocalConnection.get();
        if ((conn==null) || !conn.isValid()) {
            if (getLogger().isDebugEnabled()) {
                getLogger().debug("No valid connections found, a new one will be created"); //$NON-NLS-1$
            }
            conn = new DataSourceConnection(dataSource, readOnly);
            threadLocalConnection.set(conn);
        }
        conn.setReadOnly(readOnly);
        conn.addCaller(connectionCaller);
        return conn;
    }

    @Override
    public DataSource getDataSource() {
        return dataSource;
    }

}
