/* ----------------------------------------------------------------------------
 *     PROJECT : JPOrm
 *
 *  CREATED BY : Francesco Cina'
 *          ON : Mar 2, 2013
 * ----------------------------------------------------------------------------
 */
package com.jpattern.orm.session.writer;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Collection;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jpattern.orm.persistor.type.TypeFactory;
import com.jpattern.orm.persistor.type.TypeWrapperJdbcReady;
import com.jpattern.orm.session.PreparedStatementSetter;

/**
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Mar 2, 2013
 *
 * @author Francesco Cina'
 * @version $Revision
 */
public class PrepareStatementSetterCollectionWrapper implements PreparedStatementSetter {

    private final Logger logger = LoggerFactory.getLogger(getClass());
    private TypeFactory typeFactory;
    private Collection<?> args;

    public PrepareStatementSetterCollectionWrapper(final Collection<?> args, final TypeFactory typeFactory) {
        this.args = args;
        this.typeFactory = typeFactory;
    }

    @Override
    public void set(final PreparedStatement ps) throws SQLException {
        if (logger.isDebugEnabled()) {
            logger.debug("Query params: " + args); //$NON-NLS-1$
        }
        int index = 0;
        for (Object object : args) {
            if (object!=null) {
                TypeWrapperJdbcReady<Object, Object> typeWrapper = (TypeWrapperJdbcReady<Object, Object>) typeFactory.getTypeWrapper(object.getClass());
                typeWrapper.getJdbcIO().setValueToPreparedStatement( typeWrapper.unWrap(object) , ps, ++index);
            } else {
                ps.setObject(++index, object);
            }
        }
    }

}
